__kernel void empty() {
}

__kernel void lanes(__global unsigned *res) {
    unsigned start, lane, smi;
    asm("mov.u32 %0, %%clock;" : "=r"(start));
    asm("mov.u32 %0, %%laneid;" : "=r"(lane));
    asm("mov.u32 %0, %%smid;" : "=r"(smi));
    res[get_global_id(0)] = lane;
    res[get_global_size(0) + get_global_id(0)] = smi;
    res[2 * get_global_size(0) + get_global_id(0)] = start;
}


__kernel void shmem_latency(__global unsigned *res, __local unsigned *shmem) {
    int i, pos;
    float val;
    unsigned start, end, sum;


    for (i = 0; i < KERNEL_ITERATIONS; i++) {
	shmem[KERNEL_ITERATIONS * get_local_id(0) + i] = KERNEL_ITERATIONS * get_local_id(0) + i + 1;
    }

    pos = KERNEL_ITERATIONS * get_local_id(0);


/*
    for (i = 0; i < KERNEL_ITERATIONS; i++) {
	shmem[i * get_local_size(0) + get_local_id(0)] = (i + 1) * get_local_size(0) + get_local_id(0);
    }

    pos = get_local_id(0);
*/

    barrier(CLK_LOCAL_MEM_FENCE);

    asm("mov.u32 %0, %%clock;" : "=r"(start));
//	asm("ld.shared.f32 %0, [%1];" : "=f"(val) : "r"(shmem + pos));

#pragma unroll KERNEL_ITERATIONS
    for (i = 0; i < KERNEL_ITERATIONS; i++) {
        pos = shmem[pos];
    }
    
    asm("mov.u32 %0, %%clock;" : "=r"(end));
    sum = end - start;

    if (get_group_id(0) == 0) {
	res[get_global_id(0)] = sum;
	res[2 * get_global_size(0)] = pos;
    }
}

__kernel void shmem_bandwidth(__global unsigned *res, __local unsigned *shmem) {
    int i, pos;
    float val;
    unsigned start, end, sum;
    unsigned smi = 0;

    pos = get_local_id(0);

    asm("mov.u32 %0, %%smid;" : "=r"(smi));

    for (i = 0; i < KERNEL_ITERATIONS; i++) {
	shmem[pos] = pos;
    }

    barrier(CLK_LOCAL_MEM_FENCE);

    pos = shmem[pos];

    asm("mov.u32 %0, %%clock;" : "=r"(start));
#pragma unroll KERNEL_ITERATIONS
    for (i = 1; i < KERNEL_ITERATIONS; i++) {
        pos = shmem[pos];
    }
    asm("mov.u32 %0, %%clock;" : "=r"(end));

    barrier(CLK_LOCAL_MEM_FENCE);

    atomic_max(res + smi * 2 * get_local_size(0) + get_local_id(0), end - start);
    res[MAX_SMI * 2 * get_local_size(0) + get_global_id(0)%get_local_size(0)] = pos;

}
