#include "random123/threefry.h"

__kernel void pi(__global unsigned long *res, unsigned long n, __global unsigned long *scratch, __local unsigned long *shmem) {
    long int i;
    long hits = 0;

    int threads = get_global_size(0);
    int groups = get_num_groups(0);
    int group = get_group_id(0);
    int pos = get_global_id(0);
    int tid = get_local_id(0);

    threefry4x32_key_t k = {{pos, 0xdecafbad, 0xfacebead, 0x12345678}};
    threefry4x32_ctr_t c = {{0, 0xf00dcafe, 0xdeadbeef, 0xbeeff00d}};

    for(i = 0; i < n; i+=2) {
	union {
	    threefry4x32_ctr_t c;
	    int4 i;
	} u;
	c.v[0]++;
	u.c = threefry4x32(c, k);
	long x1 = u.i.x, y1 = u.i.y;
	long x2 = u.i.z, y2 = u.i.w;
	if ((x1*x1 + y1*y1) < (1L<<62)) {
	    hits++;
	}
	if ((x2*x2 + y2*y2) < (1L<<62)) {
	    hits++;
	}
    }

    shmem[tid] = hits;

    barrier(CLK_LOCAL_MEM_FENCE);

    for(i = get_local_size(0)/2; i>0; i >>= 1) {
	if (tid < i) shmem[tid] += shmem[tid + i];
	barrier(CLK_LOCAL_MEM_FENCE);
    }

    if (tid == 0) { 
	scratch[group] = shmem[0];
    }

    barrier(CLK_GLOBAL_MEM_FENCE);

    if (pos == 0) {
	hits = 0;
	for (i = 0; i < groups; i++)
	    hits += scratch[i];

	*res = hits;
    }

}
