import pyopencl as cl  
import sys  
import Image  
import numpy  
from time import time

  
def RoundUp(groupSize, globalSize):  
    r = globalSize % groupSize;  
    if r == 0:  
        return globalSize;  
    else:  
        return globalSize + groupSize - r;  
  
def main():  
    platforms = cl.get_platforms()
    devices = platforms[0].get_devices()
    print("Device: %s" % devices[0].name)

    data_points = 2**23 # ~8 million data points, ~32 MB data

    a = numpy.random.rand(data_points).astype(numpy.float32)
    b = numpy.random.rand(data_points).astype(numpy.float32)
    gpu_result = numpy.empty_like(a)

    time1 = time()
    cpu_result = a + b
    time2 = time()
    elapsed = (time2 - time1) * 10000000
    print("CPU execution time: %g us" % elapsed)


    imageObjects = [ 0, 0 ]  

    # create context and command queue  
    ctx = cl.create_some_context()  
    queue = cl.CommandQueue(ctx, properties=cl.command_queue_properties.PROFILING_ENABLE)

    mf = cl.mem_flags
    a_buf = cl.Buffer(ctx, mf.READ_ONLY | mf.COPY_HOST_PTR, hostbuf=a)
    b_buf = cl.Buffer(ctx, mf.READ_ONLY | mf.COPY_HOST_PTR, hostbuf=b)
    res_buf = cl.Buffer(ctx, mf.WRITE_ONLY, gpu_result.nbytes)
      
    # load the kernel source code  
    kernelFile = open("add.cl", "r")  
    kernelSrc = kernelFile.read()  
    program = cl.Program(ctx, kernelSrc).build()  
      
    # Call the kernel directly
    localWorkSize = ( 256, )
    globalWorkSize = ( data_points, )
  
    event = program.add(queue, globalWorkSize, localWorkSize, res_buf, a_buf, b_buf)
    event.wait()
    elapsed = 1e-3*(event.profile.end - event.profile.start)
    print("GPU execution time: %g us" % elapsed)

    cl.enqueue_read_buffer(queue, res_buf, gpu_result).wait()
    equal = numpy.all( cpu_result == gpu_result)

    if not equal:
        print("Results doesn't match!!")
    else:
        print("Results OK")

main()  
