import pyopencl as cl  
import sys  
import Image  
import numpy  
from time import time

  
def RoundUp(groupSize, globalSize):  
    r = globalSize % groupSize;  
    if r == 0:  
        return globalSize;  
    else:  
        return globalSize + groupSize - r;  
  
def main():  
    platforms = cl.get_platforms()
    devices = platforms[0].get_devices()
    print("Device: %s" % devices[0].name)

    imageObjects = [ 0, 0 ]  

    # create context and command queue  
    ctx = cl.create_some_context()  
    queue = cl.CommandQueue(ctx, properties=cl.command_queue_properties.PROFILING_ENABLE)
      
    # load image  
    im = Image.open('scale.jpg')  
    if im.mode != "RGBA":  
        im = im.convert("RGBA")  
    origSize = im.size 
    imgSize = origSize[0] / 4, origSize[1] / 4
    buffer = im.tostring()  
  
    # Create ouput image object  
    clImageFormat = cl.ImageFormat(cl.channel_order.RGBA, cl.channel_type.UNSIGNED_INT8)
    imageObjects[0] = cl.Image(ctx, cl.mem_flags.READ_ONLY | cl.mem_flags.COPY_HOST_PTR, clImageFormat, origSize, None, buffer)
    imageObjects[1] = cl.Image(ctx, cl.mem_flags.WRITE_ONLY, clImageFormat, imgSize)
  
    # load the kernel source code  
    kernelFile = open("scale.cl", "r")  
    kernelSrc = kernelFile.read()  
  
    # Create OpenCL program  
    program = cl.Program(ctx, kernelSrc).build()  
      
    # Call the kernel directly  
    localWorkSize = ( 16, 16 )  
    globalWorkSize = ( RoundUp(localWorkSize[0], imgSize[0]), RoundUp(localWorkSize[1], imgSize[1]) )  
  
    event = program.scale(queue, globalWorkSize, localWorkSize, imageObjects[0], imageObjects[1])
    event.wait()
    elapsed = 1e-3*(event.profile.end - event.profile.start)
    print("GPU execution time: %g us" % elapsed)
 
    # Read the output buffer back to the Host  
    buffer = numpy.zeros(imgSize[0] * imgSize[1] * 4, numpy.uint8)  
    origin = ( 0, 0, 0 )  
    region = ( imgSize[0], imgSize[1], 1 )  
      
    cl.enqueue_read_image(queue, imageObjects[1], origin, region, buffer).wait()  
      
    # Save the image to disk  
    gsim = Image.fromstring("RGBA", imgSize, buffer.tostring())  
    gsim.save('scaled.jpg')  

    time1 = time()
    res = im.resize(imgSize, Image.LINEAR)
    time2 = time()
    elapsed = (time2 - time1) * 10000000
    print("CPU execution time: %g us" % elapsed)

#    gsim = Image.fromstring("RGBA", imgSize, res.tostring())  
#    gsim.save('scaled_cpu.jpg')  

main()  
