#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>

#define BLOCK 16384
#define EXTREME 1.e+6
#define ZERO 1.e-6

int main(int argc, char *argv[]) {
    long i;
    FILE *o, *s;
    float obuf[BLOCK], sbuf[BLOCK];
    size_t res, osize, ssize;
    double avg_diff = 0, max_diff = 0, var_diff = 0, max_err, sum = 0, min = EXTREME, max = -EXTREME;
    size_t good = 0, oinv = 0, sinv = 0;
    
    if (argc < 3) {
	printf("Run %s [original] [sample]\n", argv[0]);
	exit(-1);
    }

    s = fopen(argv[1], "r");
    if (!s) {
	printf("%s not found\n", argv[1]);
	exit(-1);
    }

    o = fopen(argv[2], "r");
    if (!o) {
	printf("%s not found\n", argv[2]);
	exit(-1);
    }


    while ((!feof(s))||(!feof(o))) {
	osize = fread(obuf, sizeof(float), BLOCK, o);
	ssize = fread(sbuf, sizeof(float), BLOCK, s);
	
	if (osize != ssize) {
	    printf("Error: The sample size does not match original\n");
	    exit(-1);
	}
	
	if (!osize) break;
	
	for (i = 0; i < BLOCK; i++) {
	    double diff, err;
	    
	    if ((!isfinite(obuf[i]))||(fabsf(obuf[i]) > EXTREME)) {
		oinv++;
		continue;
	    }

	    if ((!isfinite(sbuf[i]))||(fabsf(sbuf[i]) > EXTREME)) {
		sinv++;
		continue;
	    }
	    
	    good++;
	    sum += fabsf(obuf[i]);
	    
	    
	    if (obuf[i] > max) max = obuf[i];
	    if (obuf[i] < min) min = obuf[i];
	    
	    diff = fabsf(sbuf[i] - obuf[i]);
	    if (diff > max_diff) max_diff = diff;
	    avg_diff += diff;

	    if (good > 1) {
		var_diff += (avg_diff - diff * good) * (avg_diff - diff * good) / (good * (good - 1));
	    }
/*	    
	    if (diff > ZERO) {
//	        if (fabsf(obuf[i]) > ZERO) err = diff / fabsf(obuf[i]);
	        if (fabsf(obuf[i]) > fabs(sbuf[i])) err = diff / fabsf(obuf[i]);
		else err = diff / fabsf(sbuf[i]);
	    } else err = 0;
	    if (err > max_err) max_err = err;
*/
	}
    }

    close(o);
    close(s);

    if (good < 1) {
	if (!sinv) printf("Original is probably corrupted\n");
	else printf("No good values are found in sample\n");
	exit(-1);
    }

    printf("Number of good values: % 9zu (%.2lf %%)\n", good, 100.*good/(good + oinv));
    printf("Range                : % 9.4lf (%lf - %lf)\n", max - min, min, max);
    printf("Average difference   : % 9.4lf (%.2lf %%)\n", avg_diff / good, ((sum > ZERO)?(100.*avg_diff / sum):0));
    printf("Maximal difference   : % 9.4lf (%.2lf %%)\n", max_diff, 100.*max_diff / (max - min));
    if (good > 1) {
	printf("Variance             : % 9.4lf\n", sqrt(var_diff / (good - 1)));
    }
    exit(0);
}