#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>

#define ITERATIONS 10
#define DEFAULT_SIZE 100

size_t exercise(unsigned long *res, size_t size, int iterations);
int exercise_init(const char *name, size_t size);
void exercise_free();

extern int exercise_required_alignment;

size_t get_padded_size(size_t size, size_t block_size) {
    size_t blocks = (size / block_size);
    size_t inc = blocks * block_size - size;
    if (inc) return (blocks + 1) * block_size;
    return size;
}

int main(int argc, char *argv[]) {
    int err;
    long i;
    size_t size = DEFAULT_SIZE;
    size_t iterations = ITERATIONS;
    int standard_memory = 0;
    unsigned long res;
    
    size_t runtime;
    size_t us;
    struct timeval tv1,tv2;

    char *name, fname[255];
    FILE *f;

    name = strrchr(argv[0], '/');
    if (name) name += 1;
    else name = argv[0];

    if (argc > 1) {
        size = atoi(argv[1]);
    }
    
    if (argc > 2) {
	iterations = atoi(argv[2]);
	if (!iterations) iterations = 1;
    }
    
    size *= 1000000;

    if (exercise_required_alignment > 1) {
	size = get_padded_size(size, exercise_required_alignment);
    }

    setlocale(LC_NUMERIC, "en_US");
    printf("Computing pi using %'lum throws of monte carlo, Iterations %u\n", size/1000000, iterations);

    err = exercise_init(name, size);
    if (err) return err;

    gettimeofday(&tv1, NULL);
    runtime = exercise(&res, size, iterations);
    gettimeofday(&tv2, NULL);

    if (runtime == (size_t)-1) {
	return 1;
    }

    printf("Result: %lf (should 3.14159265358979323846)\n", 4. * res / size);

    exercise_free();

    us = (tv2.tv_sec - tv1.tv_sec)*1000000 + (tv2.tv_usec - tv1.tv_usec);
    us /= iterations;
    printf("Speed: %.2lf giga-tries/sec (%lf s)\n", 0.001 * size / us, us / 1000000.);

    return 0;
}

